# Multiverse Configuration

This project is the example configuration a simulation using the "new"
configuration standard. It mainly incorporates support for the following
changes:
- [Base Class](https://almanak.atlassian.net/wiki/spaces/AE/pages/444825828/Base+Classes)
    - Agents
    - Strategies
    - Protocols
    - Metrics
    - Environments
- [Configuration Library](https://almanak.atlassian.net/wiki/spaces/AE/pages/455966746/Simulation+Configuration+Library)
- Agents running concurrently
- Multi chain support

All of these changes combined require a fairly big redesign in the
`configuration.yaml`.

The `configuration.yaml` as seen in this folder is the one the simulator will
receive. So they won't necessarily be edited by end users since this is what
will be generated by the Multiverse SDK. But the other files (see the folders
and [environment1.py](./environments/environment1/main.py)) will have to be written by the user.

Each component here supports being uploaded to the Library system (with the
exception for an `environment` for now, but we can add support for that in the
future).

For a library item to be configured you have to make sure its in a separate
folder from any other files and includes a `almanak-library.yaml`. This contains
information around the library item itself but also what dependencies this item
requires.

The dependencies that can be defined for now are limited to:
- `contracts` for Protocols
- `models` for Strategies

In the future we can add support for making library items depend on each other
(a `Strategy` might require a certain `Protocol`) and possibly add support for
limitations (think `Protocol X` can only run on `Environment Y`).

The YAML supports pulling information from different sources, like the
following:
- Files (`file://`, if specifying a folder we assume the `almanak-library.yaml`
  is available to help guide the Simulation with this local Library item)
- Library (`library://`)
- WanDB (`wandb://`)

In this example configuration we use `file://` since all of these files are made
available locally.

Some perspective on what happens with this file once its passed to the
simulator; all local files seen here are uploaded to a Google Cloud Storage
folder which will be mounted to the Simulator Instance that will use it to run a
simulation based on it. If a `library://` item has been specified the simulation
service will find it, mount it separately to the simulation instance and update
the reference to the mounted location.